'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _child_process = require('child_process');

var _r6rs = require('r6rs');

var _r6rsAsyncIo = require('r6rs-async-io');

var processes = {};
var processListeners = {};
var processBuf = {};
var processId = 0;

function makeListener(type) {
  return function (params, callback) {
    (0, _r6rs.assert)(params, 'number');
    var procId = params.value;
    var proc = processes[procId];
    if (proc == null) {
      throw new Error('Unknown process ID ' + procId);
    }
    var procListeners = processListeners[procId];
    if (type !== 'close') {
      procListeners[type].push(callback);
    }
    procListeners.close.push(function (data) {
      if (type === 'close') {
        return callback(data, true);
      }
      callback([null], true);
    });
    // Push buffer
    setTimeout(function () {
      processBuf[procId] = processBuf[procId].filter(function (v) {
        if (v[0] === 'close') {
          if (type === 'close') {
            callback(v.slice(1), true);
            return false;
          } else {
            callback([null]);
            return true;
          }
        }
        if (v[0] === type) {
          callback(v.slice(1));
          return false;
        }
        return true;
      });
    }, 0);
    return function () {
      var index = procListeners[type].indexOf(callback);
      if (index === -1) return;
      procListeners[type].splice(index, 1);
    };
  };
}

exports.default = new _r6rsAsyncIo.Library('process', {
  'process/exec': function processExec(params, callback) {
    var command = void 0,
        options = {};
    if (params.type === _r6rs.STRING) {
      command = params.value;
    } else {
      command = (0, _r6rs.toObject)(params)[0];
      options = (0, _r6rs.fromAssoc)((0, _r6rs.toObject)(params)[1]);
    }
    (0, _child_process.exec)(command, options, function (error, stdout, stderr) {
      var args = [error && [error.message, error.code, error.signal], stdout && stdout.toString('utf8'), stderr && stderr.toString('utf8')];
      callback(args, true);
    });
  },
  // To prevent cleanup failure, users must run this method with 'io-listen'
  // quite weird..
  'process/spawn': function processSpawn(params, callback) {
    var command = void 0,
        args = void 0,
        options = {};
    var paramsObj = (0, _r6rs.toObject)(params);
    command = paramsObj[0];
    if (Array.isArray(command)) {
      args = command.slice(1);
      command = command[0];
    }
    if (Array.isArray(paramsObj[1])) {
      options = (0, _r6rs.fromAssoc)(paramsObj[1]);
    }
    var proc = (0, _child_process.spawn)(command, args, options);
    var procId = processId++;
    processes[procId] = proc;
    processBuf[procId] = [];
    var procListeners = {
      close: [],
      error: [],
      stdout: [],
      stderr: []
    };
    processListeners[procId] = procListeners;
    setTimeout(function () {
      return callback([procId]);
    }, 0);
    proc.on('close', function (code, signal) {
      if (procListeners.close.length > 0) {
        procListeners.close.forEach(function (v) {
          return v([code, signal]);
        });
      }
      delete processes[procId];
      delete processListeners[procId];
      if (processBuf[procId] == null) return;
      processBuf[procId].push(['close', code, signal]);
    });
    proc.on('error', function (err) {
      if (procListeners.error.length > 0) {
        procListeners.error.forEach(function (v) {
          return v([err.message]);
        });
      }
      if (processBuf[procId] == null) return;
      processBuf[procId].push(['error', err.message]);
    });
    proc.stdout.on('data', function (data) {
      var msg = data.toString('utf-8');
      if (procListeners.stdout.length > 0) {
        procListeners.stdout.forEach(function (v) {
          return v([msg]);
        });
      }
      if (processBuf[procId] == null) return;
      processBuf[procId].push(['stdout', msg]);
    });
    proc.stderr.on('data', function (data) {
      var msg = data.toString('utf-8');
      if (procListeners.stdout.length > 0) {
        procListeners.stdout.forEach(function (v) {
          return v([msg]);
        });
      }
      if (processBuf[procId] == null) return;
      processBuf[procId].push(['stderr', msg]);
    });
    setTimeout(function () {
      return delete processBuf[procId];
    }, 12000);
    return function () {
      if (proc.connected) proc.kill();
    };
  },
  'process/onStdout': makeListener('stdout'),
  'process/onStdin': makeListener('stdin'),
  'process/onError': makeListener('error'),
  'process/onClose': makeListener('close'),
  'process/kill': function processKill(params, callback) {
    var args = (0, _r6rs.toObject)(params);
    var procId = void 0,
        code = void 0;
    if (Array.isArray(args)) {
      procId = args[0];
      code = args[1];
    } else {
      procId = args;
      code = null;
    }
    var proc = processes[procId];
    if (proc == null) {
      throw new Error('Unknown process ID ' + proc);
    }
    if (typeof code !== 'string' && code != null) {
      throw new Error('code must be string or null');
    }
    if (code == null) code = undefined;
    proc.kill(code);
    setTimeout(function () {
      callback([], true);
    }, 0);
  },
  'process/writeStdin': function processWriteStdin(params, callback) {
    var _toObject = (0, _r6rs.toObject)(params);

    var _toObject2 = _slicedToArray(_toObject, 2);

    var procId = _toObject2[0];
    var data = _toObject2[1];

    var proc = processes[procId];
    if (proc == null) {
      throw new Error('Unknown process ID ' + proc);
    }
    if (typeof data !== 'string') {
      throw new Error('data must be string');
    }
    proc.stdin.write(data, 'utf-8', function () {
      callback([], true);
    });
  }
});